# Stubs for mersad.test.util.test_base_class (Python 3)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

# Python Standard Library
import unittest
from typing import Any

class TestMersadClassicalBase(unittest.TestCase):
    BaseClass: Any = ...
    def setUp(self) -> None: ...
    def test_config(self) -> None: ...
    def test_config_reset(self) -> None: ...
    def test_process_replace_manual_key(self) -> None: ...
    def test_return_key(self) -> None: ...
    def test_config_bad_type(self) -> None: ...

class TestMainFunctionClassical(unittest.TestCase):
    main: Any = ...
    def setUp(self) -> None: ...
    def test_parent_parser(self) -> None: ...
