# Stubs for mersad.test.classical (Python 3)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

# Names in __all__ with no definition:
#   test_affine_cipher
#   test_shift_cipher
