# Stubs for mersad.util.string_analyzer (Python 3)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

# Python Standard Library
from typing import Dict
from typing import List
from typing import Set

def find_unique_letters(text: str) -> Set[str]: ...
def find_letter_indexes(text: str, letter: str) -> List[int]: ...
def map_letters_to_indexes(text: str) -> Dict[str, List[int]]: ...
