# Stubs for mersad.test.util.test_type_check (Python 3)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

# Python Standard Library
import unittest

class TestTypeCheck(unittest.TestCase):
    def test_different_types(self) -> None: ...
