# Stubs for mersad.test.util.test_crypto_math (Python 3)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

# Python Standard Library
import unittest

class TestCryptoMath(unittest.TestCase):
    def test_extended_gcd(self) -> None: ...
    def test_mod_inverse(self) -> None: ...
