# Stubs for mersad.classical.shift_cipher (Python 3)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

# Python Standard Library
from typing import Union

# Mersad Library
from mersad.util.base_class import MersadClassicalBase

def main() -> None: ...

class ShiftCipher(MersadClassicalBase):
    @staticmethod
    def _translator(text: str, **kwargs: Union[int, str, bool]) -> str: ...

def shift_cipher_translator(text: str, **kwargs: Union[int, str, bool]) -> str: ...
