# Stubs for mersad.util.crypto_math (Python 3)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

# Python Standard Library
from typing import Tuple

def extended_gcd(a: int, b: int) -> Tuple[int, int, int]: ...
def mod_inverse(a: int, b: int) -> int: ...
