# Stubs for mersad (Python 3)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

# Mersad Library
from mersad import _version as _version

# Names in __all__ with no definition:
#   classical
#   test
#   util
