# Stubs for mersad.util.string_manipulation (Python 3)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

# Python Standard Library
from typing import List

def shuffle_string(text: str, seed: int) -> str: ...
def replace_letter_by_index(text: str, letter: str, indexes: List[int]) -> str: ...
