# Stubs for mersad.test.util.test_string_manipulation (Python 3)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

# Python Standard Library
import unittest

class TestStringManipulation(unittest.TestCase):
    def test_shuffle_string(self) -> None: ...
    def test_replace_letter_by_index(self) -> None: ...
