# Stubs for mersad.test.util.test_string_analyzer (Python 3)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

# Python Standard Library
import unittest

class TestFindUniqeLetters(unittest.TestCase):
    def test(self) -> None: ...

class TestFindLetterIndexes(unittest.TestCase):
    def test(self) -> None: ...

class TestMapLettersToIndexes(unittest.TestCase):
    def test(self) -> None: ...
