# Stubs for mersad._version (Python 3)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

# Python Standard Library
from typing import Tuple

__version_info__: Tuple[int, int, int]
__version__: str
